/*
 * Copyright (C) 2009 by Aiwota Programmer
 * aiwotaprog@tetteke.tk
 *
 * This file is part of Dialektos.
 *
 * Dialektos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dialektos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dialektos.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "text_element_char_size_cache.hxx"


namespace dialektos {

namespace text_element {


namespace {
inline gunichar ucs4_to_ucs2(const gunichar ucs4) {
  return (ucs4 >> 16) ? '?' : (ucs4 & 0x0000ffff);
}
} // anonymous namespace


CharSizeCache::CharSizeCache() { initialize(); }

void CharSizeCache::initialize() {
  for (size_t i = 0; i != ArrayType::size(); ++i) {
    char_width_cache[i] = -1;
    char_height_cache[i] = -1;
    bold_char_width_cache[i] = -1;
    bold_char_height_cache[i] = -1;
  }
}

void CharSizeCache::get_char_size(gunichar uch,
    Glib::RefPtr<Pango::Layout>& layout,
    double& width, double& height, bool bold) {

  const gunichar ucs2 = ucs4_to_ucs2(uch);

  int w = get_width_cache_array(bold)[ucs2];
  int h = get_height_cache_array(bold)[ucs2];

  if (w == -1 || h == -1) {
    Glib::ustring ch(1, ucs2);
    layout->set_text(ch);
    layout->get_size(w, h);
    get_width_cache_array(bold)[ucs2] = w;
    get_height_cache_array(bold)[ucs2] = h;
  }

  width = double(w) / Pango::SCALE;
  height = double(h) / Pango::SCALE;
}

void CharSizeCache::get_char_size_from_cache(gunichar uch,
    double& width, double& height, bool bold) const {

  const gunichar ucs2 = ucs4_to_ucs2(uch);
  width = double(get_width_cache_array(bold)[ucs2]) / Pango::SCALE;
  height = double(get_height_cache_array(bold)[ucs2]) / Pango::SCALE;
}


} // namespace text_element

} // namespace dialektos
